# Author: Pramod Kumar Chikkappaiah
# Emergency Patch MAG*3.0*364 that implements a workaround solution to address
# 1. Security vulnerabilities in the IRIS httpd webserver by upgrading it to version 2.4.58 in Legacy DICOM Gateway (LDGW)
# 2. TLS 1.0/1.1 vulnerabilities in Hybrid DICOM Image Gateway (HDIG) application by enabling only TLS 1.2
Param()

# Ensures PowerShell is being run as an admin
If (-NOT ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)) {
    # Warns user and exits
    Write-Host 'This script must be ran from an Administrative PowerShell window. '`n'Exiting PowerShell' -ForegroundColor Red
    Pause    
    Stop-Transcript
	# Regular Windows PowerShell - 5.1.17763.5328
    Stop-Process -Name PowerShell
	# Below command is needed for PowerShell 7 (x64)
	Stop-Process -Name pwsh
}

#Set script wide variables
$script:patchName = "MAG3_0P364_APACHEUPGRADE_TLS"
$script:IRISRoot = 'C:\InterSystems\IRISHealth'
$script:serviceCheck = Get-Service | Where-Object { $_.Name -like 'Tomcat*' }
$script:tomcatName = "Tomcat9"
$script:regFile = @"
Windows Registry Editor Version 5.00

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders]
"SecurityProviders"="credssp.dll"

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL]
"EventLogging"=dword:00000001

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Ciphers]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\CipherSuites]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Hashes]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\KeyExchangeAlgorithms]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.0]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.0\Client]
"DisabledByDefault"=dword:00000001
"Enabled"=dword:00000000

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.0\Server]
"DisabledByDefault"=dword:00000001
"Enabled"=dword:00000000

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.1]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.1\Client]
"DisabledByDefault"=dword:00000001
"Enabled"=dword:00000000

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.1\Server]
"DisabledByDefault"=dword:00000001
"Enabled"=dword:00000000

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client]
"DisabledByDefault"=dword:00000000
"Enabled"=dword:00000001

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server]
"DisabledByDefault"=dword:00000000
"Enabled"=dword:00000001

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurityProviders\WDigest]
"Debuglevel"=dword:00000000
"Negotiate"=dword:00000000
"UTF8HTTP"=dword:00000001
"UTF8SASL"=dword:00000001
"DigestEncryptionAlgorithms"="3des,rc4"
"UseLogonCredential"=dword:00000000

"@

# Start transcript
Start-Transcript -Path "C:\Programdata\DeptOfVeteransAffairs\Logs\$($patchName).log" -Append

function Update-HTTPD {

    #Upgrade Apache
    function Upgrade-Apache {
        Write-Host 'Copying latest files' 
        Copy-Item -Path ".\ApacheUpgrade_TLS\*" -Destination "$IRISRoot\httpd\" -Recurse -Force
    }

    #Restore
    function Restore-Apache {

        Remove-Item -Path "$IRISRoot\httpd\*" -Recurse -Force
        Copy-Item -Path "$BackupFolder\*" -Destination "$IRISRoot\httpd\" -Recurse -Force

    }

    #Stop IRIS service
    function Stop-IRIS {
  
        Write-Host 'Stopping IRIS' 
        $stopIRIScmd = "$IRISRoot\bin\iris.exe stop IRIS"
        Invoke-Expression -Command $stopIRIScmd
        $apacheProcess = get-process httpd -ErrorAction SilentlyContinue
        $count = 0
        while ($apacheProcess -and $count -lt 12) {
            $secs = 60 - $count * 5
            Write-Progress -Activity "Waiting for IRIS to stop" -SecondsRemaining $secs
            Start-Sleep -Seconds 10
            $apacheProcess = get-process httpd -ErrorAction SilentlyContinue
            $count++;
        }
       
        Write-Host 'IRIS Stopped' 
    }

    #Start IRIS service
    function Start-IRIS {
        Write-Host 'Starting IRIS' 
        $startIRIScmd = "$IRISRoot\bin\iris.exe start IRIS"
        Invoke-Expression -Command $startIRIScmd
	
        $apacheProcess = get-process httpd -ErrorAction SilentlyContinue
        $count = 0
        while ($apacheProcess -and $count -lt 12) {
            $secs = 60 - $count * 5
            Write-Progress -Activity "Waiting for IRIS to start" -SecondsRemaining $secs
            Start-Sleep -Seconds 10
            $apacheProcess = get-process httpd -ErrorAction SilentlyContinue
            $count++;
        }
    
        Write-Host 'IRIS Started' 
    }

    function Print-ApacheVersion {

        Write-Host 'Apache Version is' 
        $startIRIScmd = "$IRISRoot\httpd\bin\httpd.exe -v"
        # Invoke-Expression -Command $startIRIScmd
        $apacheVersion = Invoke-Expression -Command $startIRIScmd
        Write-Host $apacheVersion 
    }
    function Update-VixConfig {
 
        $XMLfile = 'C:\VixConfig\ViXConfig.xml'
     
        [XML]$ViXConfigVar = Get-Content $XMLfile
     
     
        $CurrentVersion = $ViXConfigVar.SelectSingleNode("//void[@property='vixSoftwareVersion']").string
    
     
        if ($CurrentVersion -eq '30.345.2.8003') {
            #Backup the existing VixConfig	
            $ViXConfigVar.Save("C:\VixConfig\ViXConfig-P345_BackUp.xml")
            $ViXConfigVar.SelectSingleNode("//void[@property='vixSoftwareVersion']").string = '30.364.1.8004'
            $ViXConfigVar.Save("C:\VixConfig\ViXConfig.xml")
        } 
        else {
            #Backup the existing VixConfig	
            $ViXConfigVar.Save("C:\VixConfig\ViXConfig-P314_BackUp.xml")
            $ViXConfigVar.SelectSingleNode("//void[@property='vixSoftwareVersion']").string = '30.364.1.8004'
            $ViXConfigVar.Save("C:\VixConfig\ViXConfig.xml")
        }
    }
    
    Write-Host "Updating HTTPD`n"

    do {
        $promptOutput = Read-Host "Enter IRIS root Path. Press Enter to use default value [$($IRISRoot)]"
        if (!$promptOutput -eq "") { 
            $IRISRoot = $promptOutput
        }
        if (!(Test-Path "$IRISRoot\bin\iris.exe")) {
            Write-Host 'Invalid IRIS installation at ' $IRISRoot -ForegroundColor Red
        }
    } until (
        <# Condition that stops the loop if it returns true #>
        Test-Path "$IRISRoot\bin\iris.exe"
    )
            
    Print-ApacheVersion
    Stop-IRIS
    Upgrade-Apache

    if ($serviceCheck) {
        Update-VixConfig
    }
    Start-IRIS
    Print-ApacheVersion

    # Write completion message for HTTPD
    Write-Host "HTTPD update complete." -ForegroundColor Green
    Start-Sleep -Seconds 1
}

function Update-RegKey {
    $scriptBlockRegKey = {
        param(
            $regFile
        ) 
        $regFile | out-file $env:temp\DisableTLS.reg
        reg.exe import $env:temp\DisableTLS.reg
        Remove-Item -Path $env:temp\DisableTLS.reg
    }

    # Invoke 
    Invoke-Command -ScriptBlock $scriptBlockRegKey -ArgumentList $regFile
    
}

function Update-ServerXML {

    # Set variables used
    $serverXML = 'C:\Program Files\Apache Software Foundation\Tomcat 9.0\conf\server.xml'
    $fileTest = Test-Path -Path $serverXML
     
    # Shut down and restart Tomcat Service. Function borrowed from VIX Scripts
    function Restart-Tomcat {
        #Pre-shutdown statuses
        $tomcatpid = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty ProcessId
        $tomcatstatus = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty Status
        $tomcatstate = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty State
        Write-Output "***** Tomcat9 Service *****"
        Write-Output "Tomcat PID:  $tomcatpid"
        Write-Output $tomcatstatus
        Write-Output $tomcatstate

        #check state
        #if "Running", proceed to shutdown.  if not log and alert.
        if ($tomcatstate -eq 'Running') {
            Write-Output "$tomcatname service is running.  Stopping service..."
            $tomcatwait = 30 # number of seconds to wait for tomat to stop
            $tomcatproc = Stop-ServiceWithTimeout $tomcatname $tomcatwait
            if ($false -eq $tomcatproc) {
                Write-Output "Tomcat service did not stop in $tomcatwait seconds, killing process."
                Stop-Process -Name "$tomcatname" -Force
            }
            else {
                Write-Output "Verifying Tomcat service did stop in $tomcatwait seconds."
            }
            #verifying service stopped
            Start-Sleep -Seconds 10
            $tomcatstate2 = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty State
            Write-Output "$tomcatname is $tomcatstate2"
            if ($tomcatstate2 -eq 'Stopped') {
                Write-Output "Tomcat service successfully stopped.  Restarting service..."
                Start-Service -Name "$tomcatname"
                #verifying service started
                Start-Sleep -Seconds 10
                $tomcatstate3 = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty State
                Write-Output "$tomcatname is $tomcatstate3"
                if ($tomcatstate3 -eq 'Running') {
                    Write-Output "Tomcat service successfully restarted."
                }
            }
        }
        else {
            Write-Output "$tomcatname is not running.  That's not good."
            Write-Output "Attempting to start $tomcatname service.  Starting service..."
            Start-Service -Name "$tomcatname"
            #verifying service started
            Start-Sleep -Seconds 10
            $tomcatstate3 = Get-CimInstance -Class Win32_Service -Filter "Name LIKE '$tomcatname'" | Select-Object -ExpandProperty State
            Write-Output "$tomcatname is $tomcatstate3"
            if ($tomcatstate3 -eq 'Running') {
                Write-Output "Tomcat service successfully restarted."
            }	
        }

    }

    # Function to stop Tomcat
    function Stop-ServiceWithTimeout ([string] $name, [int] $timeoutSeconds) {
        $timespan = New-Object -TypeName System.Timespan -ArgumentList 0, 0, $timeoutSeconds
        $svc = Get-Service -Name $name
        $svc.Stop()
        try {
            $svc.WaitForStatus([ServiceProcess.ServiceControllerStatus]::Stopped, $timespan)
        }
        catch [ServiceProcess.TimeoutException] {
            return $false
        }
        return $true
    }

    if ($fileTest -eq 'true') {
        # Makes a copy of the server.xml file
        do {
            Write-Host "Creating a copy of the server.xml                                              "
            $backupTest = (Test-Path -Path 'C:\Program Files\Apache Software Foundation\Tomcat 9.0\conf\server_pre364.xml')
            Copy-Item -Path $serverXML -Destination 'C:\Program Files\Apache Software Foundation\Tomcat 9.0\conf\server_pre364.xml' -Force
			Start-Sleep -Seconds 10
        } until (
            <# Condition that stops the loop if it returns true #>
            $backupTest -eq 'true'
        )

        # Get XML content
        $xmlFile = [xml](Get-Content -Path $serverXML)
    
        # Pull data for Connector 443
        $sslSection = $xmlFile.Server.Service.connector[1]
    
        # Variables for testing connector info for Connector 443
        $protocol = $sslSection.sslProtocol
        $enabled = $sslSection.SSLEnabled
        $enabledProtocols = $sslSection.sslEnabledProtocols
    
        # Verifies SSL sections
    
        if (!($protocol -eq 'TLS')) {
            $sslSection.SetAttribute("sslProtocol", "TLS")
        }
    
        if (!($enabled -eq "true")) {
            $sslSection.SetAttribute("SSLEnabled", "true")
        }
    
        if (!($enabledProtocols -eq "TLSv1.2")) {
            $sslSection.SetAttribute("sslEnabledProtocols", "TLSv1.2")
        }

        # Saves XML file 
        $xmlFile.save($serverXML)

        Restart-Tomcat
    }
    else {
        Write-Host "server.xml could not be found!'`n'Contact support for additional help." -ForegroundColor Red
        Pause
        Stop-Transcript
        # Regular Windows PowerShell - 5.1.17763.5328
		Stop-Process -Name PowerShell
		# Below command is needed for PowerShell 7 (x64)
		Stop-Process -Name pwsh
    }
    

}

# Checks server name, if a VIX this application will exit.
if ($env:COMPUTERNAME -like "OIT*IMM*V*") {
    Write-Host "$($patchName) is for Legacy and Hybrid DICOM Image Gateways only.'`n'This script will exit" -BackgroundColor DarkRed -ForegroundColor Black
    Pause
    Stop-Transcript
    # Regular Windows PowerShell - 5.1.17763.5328
    Stop-Process -Name PowerShell
	# Below command is needed for PowerShell 7 (x64)
	Stop-Process -Name pwsh    
}
else {

    $serviceCheck = (Get-Service | Where-Object { $_.Name -like 'Tomcat*' })
    if ($null -eq $serviceCheck) {
        Write-Host "Tomcat service not found on $($env:COMPUTERNAME).`nServer is considered to be a Legacy DICOM Gateway (LDGW)." -ForegroundColor Yellow
        Update-HTTPD
        Write-Host "P364 - Emergency patch script completed."
        Read-Host "Press Enter to exit"
        Stop-Transcript
    }
    elseif ($serviceCheck.DisplayName -like '*Apache Tomcat*') {
        Write-Host "Tomcat service detected.`nServer is considered to be a Hybrid DICOM Image Gateway (HDIG)." -ForegroundColor Yellow
        Update-HTTPD
        Update-RegKey
        Update-ServerXML
		Write-Host "TLS 1.0/1.1 vulnerabilities are remediated." -ForegroundColor Green
        Write-Host "P364 - Emergency patch script completed."
        Read-Host "Press Enter to exit"
        Stop-Transcript
    }
    else {
        Write-Host "Unable to determine server type.`nPlease contact support for assistance if needed." -ForegroundColor Red
        Pause
        Stop-Transcript
        # Regular Windows PowerShell - 5.1.17763.5328
		Stop-Process -Name PowerShell
		# Below command is needed for PowerShell 7 (x64)
		Stop-Process -Name pwsh 
    }
}